class MyImgProc
{
  PImage in_img;
  PImage out_img;
  
  MyImgProc(PImage img)
  {
    in_img = img;
    out_img = img;
  }
  
  void grayscale()
  {
    PImage img = createImage(
      out_img.width, out_img.height, RGB );
    
    out_img.loadPixels();
    for ( int y = 0; y < out_img.height; y++)
    {
      for ( int x = 0; x < out_img.width; x++)
      {
        int pos = x + y*out_img.width;
        color c = out_img.pixels[pos];
        float r = red( c );
        float g = green( c );
        float b = blue( c );
        float gray = 0.3 * r + 0.59 * g + 0.11 * b;
        img.pixels[pos] = color(gray);
      }
    }
    img.updatePixels();
    out_img = img;
  }
  
  void contrast(int in_min, int in_max, int out_min, int out_max)
  {
    float[] lut = new float[256];
    for ( int i = 0; i < 256; i++)
    {
      if ( i <= in_min )
      {
        lut[i] = (float)out_min;
      }
      else if ( i >= in_max)
      {
        lut[i] = (float)out_max;
      }
      else
      {
        float d = (float)(out_max - out_min) / 
          (float)(in_max - in_min);
        lut[i] = (i - in_min) * d + out_min;
      }
    }
    
    PImage img = createImage(
      out_img.width, out_img.height, RGB );
      
    out_img.loadPixels();
    for ( int y = 0; y < out_img.height; y++)
    {
      for ( int x = 0; x < out_img.width; x++)
      {
        int pos = x + y*out_img.width;
        color c = out_img.pixels[pos];
        float r = red( c );
        float g = green( c );
        float b = blue( c );
        img.pixels[pos] = 
          color(lut[(int)r], lut[(int)g], lut[(int)b]);
      }
    }
    img.updatePixels();
    out_img = img;    
  }
  
  color getPixel(PImage img, int x, int y)
  {
    if ( x < 0 )
    {
      x = 0;
    }
    if ( x >= img.width )
    {
      x = img.width - 1;
    }
    if ( y < 0 )
    {
      y = 0;
    }
    if ( y >= img.height )
    {
      y = img.height - 1;
    }
    int pos = x + y * img.width;
    color c = img.pixels[pos];
    
    return c;
  }
  
  void blur(int[] mask_mt)
  {
    int sum = 0;
    int len = mask_mt.length;
    for ( int i = 0; i < len; i++ )
    {
      sum += mask_mt[i];
    }
    int d = (int)Math.floor( Math.sqrt( len ) / 2.0 );
    
    PImage img = createImage(
      out_img.width, out_img.height, RGB );
    
    out_img.loadPixels();
    
    for ( int y = 0; y < out_img.height; y++)
    {
      for ( int x = 0; x < out_img.width; x++)
      {
        float r = 0.0;
        float g = 0.0;
        float b = 0.0;
        int mask_ix = 0;
        for ( int yy = -d; yy <= d; yy++ )
        {
          for ( int xx = -d; xx <= d; xx++ )
          {
            color c = getPixel( out_img, x + xx, y + yy );
            r += red( c ) * mask_mt[mask_ix];
            g += green( c ) * mask_mt[mask_ix];
            b += blue( c ) * mask_mt[mask_ix];
            mask_ix++;
          }
        }
        r /= (float)sum;
        g /= (float)sum;
        b /= (float)sum;
        int pos = x + y * out_img.width;
        img.pixels[pos] = color( r, g, b );
      }
    }
    img.updatePixels();
    
    out_img = img;
  }
  
  void edge(int[] mt_x, int[] mt_y)
  {
    int sum = 0;
    int len = mask_x.length;
    for ( int i = 0; i < len; i++ )
    {
      sum += abs(mask_x[i]);
    }
    int d = (int)Math.floor( Math.sqrt( len ) / 2.0 );
    
    PImage img =
      createImage( out_img.width, out_img.height, RGB );
  
    out_img.loadPixels();
    
    for ( int y = 0; y < out_img.height; y++)
    {
      for ( int x = 0; x < out_img.width; x++)
      {
        float r_x = 0.0;
        float r_y = 0.0;
        float g_x = 0.0;
        float g_y = 0.0;
        float b_x = 0.0;
        float b_y = 0.0;
        int mask_ix = 0;
        for ( int yy = -d; yy <= d; yy++ )
        {
          for ( int xx = -d; xx <= d; xx++ )
          {
            color c = getPixel( out_img, x + xx, y + yy );
            r_x += red( c ) * mask_x[mask_ix];
            r_y += red( c ) * mask_y[mask_ix];
            g_x += green( c ) * mask_x[mask_ix];
            g_y += green( c ) * mask_y[mask_ix];
            b_x += blue( c ) * mask_x[mask_ix];
            b_y += blue( c ) * mask_y[mask_ix];
            mask_ix++;
          }
        }
        float r = sqrt( (r_x * r_x + r_y * r_y) / sum );
        float g = sqrt( (g_x * g_x + g_y * g_y) / sum );
        float b = sqrt( (b_x * b_x + b_y * b_y) / sum );
        int pos = x + y * out_img.width;
        img.pixels[pos] = color( r, g, b );
      }
    }
    img.updatePixels();
    
    out_img = img;    
  }

  PImage getInImg()
  {
    return in_img;
  }
  
  PImage getOutImg()
  {
    return out_img;
  }
}


